@@ -1,9 +1,18 @@
 # Copyright (c) 2007-2008 Roy Marples <roy@marples.name>
 # All rights reserved. Released under the 2-clause BSD license.
 
+_ip()
+{
+	if [ -x /bin/ip ]; then
+		echo /bin/ip
+	else
+		echo /sbin/ip
+	fi
+}
+
 iproute2_depend()
 {
-	program /sbin/ip
+	program $(_ip)
 	provide interface
 	after ifconfig
 }
@@ -160,7 +169,8 @@ _add_route()
 
 	# We cannot use a metric if we're using a nexthop
 	if ! ${have_metric} && \
-		[ -n "${metric}" -a -z "${cmd##* nexthop }" ]
+		[ -n "${metric}" -a \
+			"${cmd##* nexthop }" = "$cmd" ]
 	then
 		cmd="${cmd} metric ${metric}"
 	fi
