@@ -1,10 +1,10 @@
 /*
-   librc-misc.c
-   rc misc functions
-   */
+  librc-misc.c
+  rc misc functions
+*/
 
 /*
- * Copyright 2007-2008 Roy Marples <roy@marples.name>
+ * Copyright (c) 2007-2008 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -29,15 +29,17 @@
  * SUCH DAMAGE.
  */
 
+#include <sys/file.h>
 #include <sys/types.h>
 #include <sys/utsname.h>
 
 #ifdef __linux__
-#include <sys/sysinfo.h>
-#include <regex.h>
+#  include <sys/sysinfo.h>
+#  include <regex.h>
 #endif
 
 #include <ctype.h>
+#include <fcntl.h>
 #include <limits.h>
 #include <signal.h>
 #include <stdio.h>
@@ -48,14 +50,15 @@
 #include "einfo.h"
 #include "rc.h"
 #include "rc-misc.h"
+#include "version.h"
 
 #define PROFILE_ENV     RC_SYSCONFDIR "/profile.env"
-#define SYS_WHITELIST   RC_LIBDIR "/conf.d/env_whitelist"
+#define SYS_WHITELIST   RC_LIBEXECDIR "/conf.d/env_whitelist"
 #define USR_WHITELIST   RC_SYSCONFDIR "/conf.d/env_whitelist"
 #define RC_CONF         RC_SYSCONFDIR "/rc.conf"
 #define RC_CONF_OLD     RC_SYSCONFDIR "/conf.d/rc"
 
-#define PATH_PREFIX     RC_LIBDIR "/bin:/bin:/sbin:/usr/bin:/usr/sbin"
+#define PATH_PREFIX     RC_LIBEXECDIR "/bin:/bin:/sbin:/usr/bin:/usr/sbin"
 
 static RC_STRINGLIST *rc_conf = NULL;
 
@@ -219,7 +222,8 @@ env_config(void)
 		free(e);
 	}
 
-	setenv("RC_LIBDIR", RC_LIBDIR, 1);
+	setenv("RC_VERSION", VERSION, 1);
+	setenv("RC_LIBEXECDIR", RC_LIBEXECDIR, 1);
 	setenv("RC_SVCDIR", RC_SVCDIR, 1);
 	setenv("RC_TMPDIR", RC_SVCDIR "/tmp", 1);
 	setenv("RC_BOOTLEVEL", RC_LEVEL_BOOT, 1);
@@ -242,7 +246,8 @@ env_config(void)
 	if (sys)
 		setenv("RC_SYS", sys, 1);
 
-	/* Some scripts may need to take a different code path if Linux/FreeBSD, etc
+	/* Some scripts may need to take a different code path if
+	   Linux/FreeBSD, etc
 	   To save on calling uname, we store it in an environment variable */
 	if (uname(&uts) == 0)
 		setenv("RC_UNAME", uts.sysname, 1);
@@ -270,30 +275,56 @@ signal_setup(int sig, void (*handler)(int))
 	return sigaction(sig, &sa, NULL);
 }
 
+int
+svc_lock(const char *applet)
+{
+	char file[PATH_MAX];
+	int fd;
+
+	snprintf(file, sizeof(file), RC_SVCDIR "/exclusive/%s", applet);
+	fd = open(file, O_WRONLY | O_CREAT | O_NONBLOCK, 0664);
+	if (fd == -1)
+		return -1;
+	if (flock(fd, LOCK_EX | LOCK_NB) == -1) {
+		close(fd);
+		return -1;
+	}
+	return fd;
+}
+
+int
+svc_unlock(const char *applet, int fd)
+{
+	char file[PATH_MAX];
+
+	snprintf(file, sizeof(file), RC_SVCDIR "/exclusive/%s", applet);
+	close(fd);
+	unlink(file);
+	return -1;
+}
+
 pid_t
 exec_service(const char *service, const char *arg)
 {
-	char *file;
-	char fifo[PATH_MAX];
+	char *file, sfd[32];
+	int fd;
 	pid_t pid = -1;
 	sigset_t full;
 	sigset_t old;
 	struct sigaction sa;
 
+	fd = svc_lock(basename_c(service));
+	if (fd == -1)
+		return -1;
+	
 	file = rc_service_resolve(service);
 	if (!exists(file)) {
 		rc_service_mark(service, RC_SERVICE_STOPPED);
+		svc_unlock(basename_c(service), fd);
 		free(file);
 		return 0;
 	}
-
-	/* We create a fifo so that other services can wait until we complete */
-	snprintf(fifo, sizeof(fifo), RC_SVCDIR "/exclusive/%s",
-		 basename_c(service));
-	if (mkfifo(fifo, 0600) != 0 && errno != EEXIST) {
-		free(file);
-		return -1;
-	}
+	snprintf(sfd, sizeof(sfd), "%d", fd);
 
 	/* We need to block signals until we have forked */
 	memset(&sa, 0, sizeof (sa));
@@ -316,18 +347,20 @@ exec_service(const char *service, const char *arg)
 		sigprocmask(SIG_SETMASK, &old, NULL);
 
 		/* Safe to run now */
-		execl(file, file, arg, (char *) NULL);
+		execl(file, file, "--lockfd", sfd, arg, (char *) NULL);
 		fprintf(stderr, "unable to exec `%s': %s\n",
-			file, strerror(errno));
-		unlink(fifo);
+		    file, strerror(errno));
+		svc_unlock(basename_c(service), fd);
 		_exit(EXIT_FAILURE);
 	}
 
-	if (pid == -1)
+	if (pid == -1) {
 		fprintf(stderr, "fork: %s\n",strerror (errno));
+		svc_unlock(basename_c(service), fd);
+	} else
+		fcntl(fd, F_SETFD, fcntl(fd, F_GETFD, 0) | FD_CLOEXEC);
 
 	sigprocmask(SIG_SETMASK, &old, NULL);
-
 	free(file);
 	return pid;
 }
